@@ -35,7 +35,7 @@ from data_science_helper.model import neg_bagging_fraction__lgb_model as nbf_lgb
 from data_science_helper.model import scale_pos_weight__lgb_model as spw_lgb_model
 from data_science_helper.model import custom_bagging__lgb_model as cb_lgb_model
 
-
+from sklearn.metrics import f1_score
 
 #import src.Prj_Core.core_helper.model.neg_bagging_fraction__lgb_model as nbf_lgb_model
 #import src.Prj_Core.core_helper.model.scale_pos_weight__lgb_model as spw_lgb_model
@@ -64,11 +64,15 @@ def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=
         model , y_prob_uno   = spw_lgb_model.modelar(**args)
     if (strategy=="custom_bagging__lgb_model"):
         model , y_prob_uno   = cb_lgb_model.modelar(X_train,y_train,X_test,y_test,url)
+        
+    umbral_opt = get_opt_threshold(y_test,y_prob_uno)
 
-    kpis = generar_reporte(model,y_prob_uno,X_test,y_test,url,print_consola)
+    kpis = generar_reporte(model,y_prob_uno,X_test,y_test,None,url,print_consola)
+    kpis_opt = generar_reporte(model,y_prob_uno,X_test,y_test,umbral_opt,url,print_consola)
+    
     print("Time elapsed: ", time.time() - start)
     
-    return model , y_prob_uno  , kpis
+    return model , y_prob_uno  , kpis , kpis_opt , umbral_opt
 
 
 def modelar_clasificacion_binaria_rscv(strategy, X_train,y_train=None,X_test=None,y_test=None,score_rs=None,params=None,param_dist=None, n_iter=None,n_jobs=None,url=None,print_consola=True):
@@ -89,13 +93,42 @@ def modelar_clasificacion_binaria_rscv(strategy, X_train,y_train=None,X_test=Non
     return results, model , predicted_probas, params, best_params ,kpis
     
 
-def generar_reporte(model,y_prob_uno, X_test, y_test,url,print_consola):    
-    kpis = hp.print_kpis_rendimiento_modelo(y_test,y_prob_uno,url,print_consola)   
+def generar_reporte(model,y_prob_uno, X_test, y_test,umbral_opt,url,print_consola): 
+     
+    kpis = hp.print_kpis_rendimiento_modelo(y_test,y_prob_uno,umbral_opt,url,print_consola)   
+  
     if  isinstance(model, list)==False:
         if (print_consola):
             hp.print_shap_plot(model, X_test, url)      
     g.generate_summary_evaluation(X_test,y_prob_uno,y_test,url) 
+    
+    
+    
     return kpis
+
+
+def get_opt_threshold(y_test=None,y_pred_prob=None):
+    thresholds = np.arange(0.0, 1.0, 0.0001)
+    score = np.zeros(shape=(len(thresholds)))
+    print('Length of sequence: {}'.format(len(thresholds)))
+
+    # Fit the model
+    for index, elem in enumerate(thresholds):
+        # Corrected probabilities
+        y_pred = (y_pred_prob > elem).astype('int')
+        #print(y_pred_prob)
+        # Calculate the f-score
+        score[index] = f1_score(y_test, y_pred)
+        #score[index] = average_precision_score(y_test, y_pred_prob)
+        #break
+
+
+    # Find the optimal threshold
+    index = np.argmax(score)
+    thresholdOpt = round(thresholds[index], ndigits = 4)
+    scoreOpt = round(score[index], ndigits = 4)
+    print('Best Threshold: {} with F-Score: {}'.format(thresholdOpt, scoreOpt))
+    return thresholdOpt
     
     
 def predecir_clasificacion_binaria(model, X=None, umbral=0.5):
@@ -118,7 +151,7 @@ def predecir_clasificacion_binaria(model, X=None, umbral=0.5):
 
 
 def get_result_df(KPIs_list):
-    df = pd.DataFrame(columns=["Macro Región","Modelo","T Train","T Test",'Precision', 'Recall', 'F1','Average Precision','ROC AUC'])
+    df = pd.DataFrame(columns=["Macro Región","Modelo","T Train","T Test",'Precision', 'Recall', 'F1','Average Precision','ROC AUC',"umbral_opt",'Precision_opt', 'Recall_opt', 'F1_opt','Average Precision opt','ROC AUC opt'])
     for idx, result  in enumerate(KPIs_list):
         mr = result[0]
         mod = result[1]
@@ -127,7 +160,9 @@ def get_result_df(KPIs_list):
         t_test = result[3]
         
         Ks = result[4]
-        result_rd = [mr,mod,t_train,t_test]+[round(num, 3) for num in list(Ks)]
+        umbral_opt = result[5]
+        Ks_opt = result[6]
+        result_rd = [mr,mod,t_train,t_test]+[round(num, 3) for num in list(Ks)]+[umbral_opt]+[round(num_opt, 3) for num_opt in list(Ks_opt)]
         df.loc[idx] = result_rd
 
         #df.loc[i] = list(KPIs)
