@@ -159,10 +159,10 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
         'bias': bias,
         'overfitting':with_bias,
         'mean_train_score':mean_train_score,
-        'std_train_score':std_valid_score,  
+        'std_train_score':std_train_score,  
         
         'mean_valid_score':mean_valid_score,
-        'std_valid_score':std_train_score,       
+        'std_valid_score':std_valid_score,       
           
     }   
     
